import React, { Component } from 'react';
import { DialogFooterProps, ModalCloseReason } from '@gravity-ui/uikit';
import { Decorator, FORM_ERROR, FormApi, FormState, ValidationErrors } from 'final-form';
import { FieldInputProps, FieldMetaState, FormProps } from 'react-final-form';
import { Calculation, Updates } from 'final-form-calculate';
import { TextControlProps } from '../TextControl/TextControl';
import { TextAreaControlProps } from '../TextAreaControl/TextAreaControl';
import { SelectControlProps } from '../SelectControl/SelectControl';
import { TumblerControlProps } from '../TumblerControl/TumblerControl';
import { PlainTextProps } from '../PlainText/PlainText';
import { CheckBoxControlProps } from '../CheckBoxControl/CheckBoxControl';
import { CustomBlockProps } from '../CustomBlock/CustomBlock';
import { RadioButtonControlProps } from '../RadioButtonControl/RadioButtonControl';
import { EditableListProps } from '../EditableList/EditableList';
import { EditableManyListsProps } from '../EditableManyLists/EditableManyLists';
import { MultiTextControlProps } from '../MultiTextControl/MultiTextControl';
import { TabFieldProps, TabItem } from '../TabField/TabField';
import './Dialog.scss';
import { TabFieldVerticalProps } from '../TabField/TabFieldVertical';
import { ControlProps, ControlStaticApi, DistributiveOmit, TabControlStaticApi, ValidatorType } from '../types';
declare type FieldName = string;
export interface ControlField<T extends string = string, V = any, E = Record<string, any>, FormValues = unknown> {
    name: string;
    type: T;
    fullWidth?: boolean;
    initialValue?: V;
    extras?: E | ((allValues: FormValues, options: {
        form: FormApi<any>;
        input: FieldInputProps<V>;
        field: ControlField<T, V, E, FormValues>;
    }) => E);
    touched?: boolean;
    visibilityCondition?: {
        when: FieldName;
        isActive: (v: any) => boolean;
    };
    className?: string;
    required?: boolean;
    caption?: string;
    tooltip?: React.ReactNode;
    warning?: React.ReactNode;
    validator?: ValidatorType<V>;
    validateFields?: string[];
    subscribers?: Updates;
    onChange?: (value: V, oldValue: V, field: ControlField<T, V>, fullName: string, form: FormApi<any>, formValues: FormState<any>) => void;
    error?: () => void;
}
export declare type TabbedField<FieldT, TabData = any, K extends string = string> = {
    name: string;
    type: K;
    fields: Array<DialogField<FieldT>>;
    size?: string;
    title?: string;
    getTitle?: (data: TabData) => string;
    isRemovable?: (position: number) => boolean;
    renderControls?: (data: TabData, onCreate: (active?: boolean) => void, onRemove?: () => void) => React.ReactNode;
    visibilityCondition?: {
        when: any;
        isActive: (v: any) => boolean;
    };
    subscribers?: undefined;
} & MultipleTabField<TabData>;
export declare type MultipleTabField<TabData> = {
    multiple?: false;
    onCreateTab?: undefined;
} | {
    multiple: true;
    onCreateTab?: (data: TabData, opts: CreateTabOptions) => TabData;
};
interface CreateTabOptions {
    userOptions: any;
}
declare type SectionType<FieldT> = {
    section: string;
    fields?: FieldT[] | SeparatorType;
    wrapTo?: (sectionNode: React.ReactNode) => React.ReactNode;
    collapsible?: boolean;
    initialCollapsed?: boolean;
    type?: never;
};
declare type SeparatorType = {
    separator: boolean;
    type?: never;
    name?: never;
    subscribers?: never;
};
declare type DialogField<FieldT> = FieldT | SectionType<FieldT> | SeparatorType;
declare type FieldsType<TabT extends TabbedField<FieldT>, FieldT> = Array<TabT> | Array<DialogField<FieldT>>;
export declare type DeepPartial<T> = T extends object ? {
    [K in keyof T]?: DeepPartial<T[K]>;
} : T;
export interface DFDialogProps<FormValues = Record<string, any>, InitialFormValues = DeepPartial<FormValues>, TabT extends TabbedField<FieldT> = any, FieldT = any> {
    className?: string;
    visible: boolean;
    fields: FieldsType<TabT, FieldT>;
    onAdd: (form: FormApi<FormValues, InitialFormValues>) => Promise<{
        validationErrors?: ValidationErrors & {
            [FORM_ERROR]?: any;
        };
    } | undefined | void>;
    onClose?: (form: FormApi<FormValues, InitialFormValues>) => void;
    validate?: (values: FormValues) => ValidationErrors | Promise<ValidationErrors> | undefined;
    onActiveTabChange?: (tab: string) => void;
    size?: 's' | 'm' | 'l';
    initialValues?: InitialFormValues;
    modal?: boolean;
    pristineSubmittable?: boolean;
    isApplyDisabled?: (values: FormState<FormValues>) => boolean;
    headerProps?: {
        title?: React.ReactNode;
        insertAfter?: React.ReactNode;
        insertBefore?: React.ReactNode;
    };
    footerProps?: {
        hidden?: boolean;
        textApply?: React.ReactNode;
        textCancel?: string;
        content?: React.ReactNode;
        propsButtonCancel?: DialogFooterProps['propsButtonCancel'];
        propsButtonApply?: DialogFooterProps['propsButtonApply'];
    };
    waitingMessage?: React.ReactNode;
    virtualized?: boolean;
    formId?: string;
    /**
     * The property allows to pass additional properties which usually are not required
     * but might be useful in some special cases
     * See https://final-form.org/docs/react-final-form/types/FormProps
     */
    formExtras?: Partial<FormProps<FormValues, InitialFormValues>>;
    disableFormReset?: boolean;
}
interface State<FormValues> {
    defaultValues?: FormValues;
    tabsCount: number;
    activeTabId?: string;
    verticalTabs?: boolean | null;
}
declare type OperateOnTabsCb = (tabInfo: OperateTabInfo) => void;
interface OperateTabInfo {
    id: string;
    error?: string;
    name: string;
    multiple?: boolean;
    index: number;
    title: string;
    isRemovable: (position: number) => boolean;
    renderControls?: TabbedField<any>['renderControls'];
    visibilityCondition?: TabbedField<any>['visibilityCondition'];
}
declare class Dialog<FormValues = Record<string, any>, InitialFormValues = DeepPartial<FormValues>, TabT extends TabbedField<FieldT> = any, FieldT extends ControlField = never> extends Component<DFDialogProps<FormValues, InitialFormValues, TabT, FieldT>, State<FormValues>> {
    static defaultProps: {
        pristineSubmittable: boolean;
        modal: boolean;
        size: string;
        headerProps: {};
        footerProps: {
            textApply: string;
            textCancel: string;
        };
        onClose: () => void;
    };
    static FINAL_FORM_SUBSCRIPTIONS: {
        submitting: boolean;
        pristine: boolean;
        invalid: boolean;
        dirtySinceLastSubmit: boolean;
        hasValidationErrors: boolean;
        hasSubmitErrors: boolean;
    };
    /**
     * @param {string} controlId
     * @param {function} controlComponent
     * @param {{
     *     suppressDuplicateWarning: boolean | undefined,
     * } | undefined} opts
     */
    static registerControl<K extends string, V, P extends {
        value?: any;
        onChange?: any;
    }>(controlId: K, controlComponent: React.ComponentType<P> & ControlStaticApi<V>, { suppressDuplicateWarning }?: {
        suppressDuplicateWarning?: boolean;
    }): void;
    static registerTabControl<K extends string>(controlId: K, tabComponent: React.ComponentType<TabFieldProps> & TabControlStaticApi, opts?: {
        suppressDuplicateWarning?: boolean;
    }): void;
    static isEmptyField<F extends ControlField, V>(field: F, value: V): boolean;
    static getDefaultValues<T extends TabbedField<F>, F>(fields?: FieldsType<T, F>): Record<string, any>;
    static makeRequiredValidator<F extends ControlField, V>(field: F): (value: V) => string | undefined;
    static getDerivedStateFromProps<TabT extends TabbedField<FieldT>, FieldT extends ControlField, FormValues>(props: DFDialogProps<FormValues, unknown, TabT, FieldT>, prevState: State<FormValues>): Partial<State<FormValues>> | null;
    static hasTabs<TabT extends TabbedField<FieldT>, FieldT extends ControlField>(fields: FieldsType<TabT, FieldT>): fields is Array<TabT>;
    static getFirstTabId<FieldT>(fields: Array<TabbedField<FieldT>>, values: any): string;
    static getTabsOrientation<TabT extends TabbedField<FieldT>, FieldT>(fields: FieldsType<TabT, FieldT>): false | null | undefined;
    static renderField<FieldT extends ControlField, Value = any>(field: FieldT, input: FieldInputProps<Value>, meta: FieldMetaState<Value>): React.JSX.Element;
    static renderSeparator(_field: SeparatorType, index: number): React.JSX.Element;
    static renderLabel(field: Pick<ControlField, 'name' | 'type' | 'required' | 'caption' | 'tooltip'>): any;
    static operateOnTabs<TabT extends TabbedField<FieldT, TabData>, FieldT, TabData = any>({ fields, values, errors }: {
        fields: Array<TabT>;
        values: any;
        errors: ValidationErrors;
    }, callback: OperateOnTabsCb): void;
    static collectSubscribers<TabT extends TabbedField<FieldT>, FieldT extends ControlField>(prefix: string | undefined, fields: FieldsType<TabT, FieldT>, dst?: Array<{
        name: Calculation['field'];
        subscribers: Calculation['updates'];
    }>): {
        name: Calculation['field'];
        subscribers: Calculation['updates'];
    }[];
    state: State<FormValues>;
    form?: FormApi<FormValues, InitialFormValues>;
    constructor(props: DFDialogProps<FormValues, InitialFormValues, TabT, FieldT>);
    componentDidUpdate(_prevProps: unknown, prevState: State<FormValues>): void;
    onClose: (_event?: unknown, reason?: ModalCloseReason | 'closeButtonClick') => void;
    _isApplyDisabled(formState: FormState<FormValues, InitialFormValues>): boolean;
    isApplyDisabled(formState: FormState<FormValues, InitialFormValues>): boolean;
    onApply: () => Promise<void | (import("final-form").AnyObject & {
        "FINAL_FORM/form-error"?: any;
    }) | null>;
    renderFieldGroup(field: FieldT, fieldPrefix?: string): React.JSX.Element;
    renderSection(field: SectionType<FieldT>, index: number, fieldPrefix?: string): React.JSX.Element;
    renderFields(fields: Array<DialogField<FieldT>>, fieldPrefix?: string): React.JSX.Element[];
    renderFormError(): React.JSX.Element | null;
    isActiveTab: (tabValues: TabT) => boolean;
    renderMultiTabContent(tab: TabT): React.JSX.Element;
    renderSingleTabContent(tab: TabT): false | React.JSX.Element;
    onActiveTabChange: (tab: string) => void;
    isVisibleTab(tabAttributes: OperateTabInfo, values: FormValues): boolean;
    prepareTabs(form: FormApi<FormValues, InitialFormValues>, formState: FormState<FormValues, InitialFormValues>): {
        tabItems: TabItem[];
        createTab: (fieldName: string, active?: boolean, options?: {
            userOptions?: any;
        }) => void;
        removeTab: (tabItemToDelete: TabItem) => void;
    };
    renderTabs: ({ form, ...formState }: {
        form: FormApi<FormValues, InitialFormValues>;
    } & FormState<FormValues, InitialFormValues>) => React.JSX.Element;
    renderTabbedFields(): React.JSX.Element;
    hasTabs(): boolean;
    renderDialogContent: (handleSubmit: () => void, form: FormApi<FormValues, InitialFormValues>) => React.JSX.Element;
    renderPageDialog: ({ handleSubmit, form, }: {
        handleSubmit: () => void;
        form: FormApi<FormValues, InitialFormValues>;
    }) => React.ReactNode;
    renderModalDialog: ({ handleSubmit, form, }: {
        handleSubmit: () => void;
        form: FormApi<FormValues, InitialFormValues>;
    }) => React.ReactNode;
    getFieldsCalculator: () => Decorator<any, any> | undefined;
    render(): React.JSX.Element;
}
export declare type RegisteredDialogTabField<Key extends string, P extends TabFieldProps, FieldT> = TabbedField<FieldT, any, Key> & Omit<P, keyof TabbedField<unknown> | 'setActiveTab' | 'tabItems'>;
export declare type DFDialogTabField<FieldT> = RegisteredDialogTabField<'tab', TabFieldProps, FieldT> | RegisteredDialogTabField<'tab-vertical', TabFieldVerticalProps, FieldT>;
export declare const DeprecatedDialog: typeof Dialog;
export declare type RegisteredDialogField<Key extends string, P extends {
    value?: any;
}, FormValues> = ControlField<Key, P['value'], DistributiveOmit<P, keyof ControlProps<any>>, FormValues>;
export declare type DFDialogField<FormValues = any, AdditionalControlFields = never> = SectionType<DFDialogControlField<FormValues> | AdditionalControlFields | SeparatorType> | SeparatorType | DFDialogControlField<FormValues> | AdditionalControlFields;
declare type DFDialogControlField<FormValues = any> = RegisteredDialogField<'text', TextControlProps, FormValues> | RegisteredDialogField<'textarea', TextAreaControlProps, FormValues> | RegisteredDialogField<'select', SelectControlProps, FormValues> | RegisteredDialogField<'tumbler', TumblerControlProps, FormValues> | RegisteredDialogField<'block', CustomBlockProps, FormValues> | RegisteredDialogField<'plain', PlainTextProps, FormValues> | RegisteredDialogField<'checkbox', CheckBoxControlProps, FormValues> | RegisteredDialogField<'radio', RadioButtonControlProps, FormValues> | RegisteredDialogField<'editable-list', EditableListProps<any>, FormValues> | RegisteredDialogField<'multi-editable-lists', EditableManyListsProps<any>, FormValues> | RegisteredDialogField<'multi-text', MultiTextControlProps, FormValues>;
export declare function DFDialog<FormValues, InitialValuesType = DeepPartial<FormValues>>(props: DFDialogProps<FormValues, InitialValuesType, DFDialogTabField<DFDialogField<FormValues>>, DFDialogField<FormValues>>): React.JSX.Element;
export declare const registerDialogControl: typeof Dialog.registerControl;
export declare const registerDialogTabControl: typeof Dialog.registerTabControl;
export {};
