import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _inherits from "@babel/runtime/helpers/inherits";
import _createSuper from "@babel/runtime/helpers/createSuper";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
var _excluded = ["form"],
  _excluded2 = ["id"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { Component, Fragment } from 'react';
import _reduce from 'lodash/reduce';
import _find from 'lodash/find';
import _findIndex from 'lodash/findIndex';
import _isEqual from 'lodash/isEqual';
import _get from 'lodash/get';
import _isEmpty from 'lodash/isEmpty';
import _map from 'lodash/map';
import { Dialog as CommonDialog, Popover } from '@gravity-ui/uikit';
import { dfCN } from '../../helpers/cn';
import { FORM_ERROR } from 'final-form';
import { Field, Form, FormSpy } from 'react-final-form';
import arrayMutators from 'final-form-arrays';
import { FieldArray } from 'react-final-form-arrays';
import createDecorator from 'final-form-calculate';
import { OnChange } from 'react-final-form-listeners';
import { InfoIcon, TooltipIcon, WarningIcon } from '../Icon/Icon';
import TextControl from '../TextControl/TextControl';
import TextAreaControl from '../TextAreaControl/TextAreaControl';
import { SelectControl } from '../SelectControl/SelectControl';
import TumblerControl from '../TumblerControl/TumblerControl';
import PlainText from '../PlainText/PlainText';
import CheckBoxControl from '../CheckBoxControl/CheckBoxControl';
import Condition from '../Condition/Condition';
import CustomBlock from '../CustomBlock/CustomBlock';
import RadioButtonControl from '../RadioButtonControl/RadioButtonControl';
import EditableList from '../EditableList/EditableList';
import EditableManyLists from '../EditableManyLists/EditableManyLists';
import MultiTextControl from '../MultiTextControl/MultiTextControl';
import TabField from '../TabField/TabField';
import { checkTabId, getTabId, applyFunctions } from './utils';
import { useStableEventHandler } from '../../helpers/useStableEventHendler';
import './Dialog.css';
import TabFieldVertical from '../TabField/TabFieldVertical';
import i18n from './i18n';
import { CollapsibleSection } from '../CollapsibleSection/CollapsibleSection';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
var bDialog = dfCN('dialog');
var bPage = dfCN('page-dialog');
function composeValidators() {
  for (var _len = arguments.length, validators = new Array(_len), _key = 0; _key < _len; _key++) {
    validators[_key] = arguments[_key];
  }
  return /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(value, allValues, state) {
      var _i, _validators, validate, res;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            _i = 0, _validators = validators;
          case 1:
            if (!(_i < _validators.length)) {
              _context.next = 17;
              break;
            }
            validate = _validators[_i];
            _context.prev = 3;
            _context.next = 6;
            return validate(value, allValues, state);
          case 6:
            res = _context.sent;
            if (!res) {
              _context.next = 9;
              break;
            }
            return _context.abrupt("return", res);
          case 9:
            _context.next = 14;
            break;
          case 11:
            _context.prev = 11;
            _context.t0 = _context["catch"](3);
            return _context.abrupt("return", 'Validation exception: ' + _context.t0);
          case 14:
            _i++;
            _context.next = 1;
            break;
          case 17:
            return _context.abrupt("return", undefined);
          case 18:
          case "end":
            return _context.stop();
        }
      }, _callee, null, [[3, 11]]);
    }));
    return function (_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  }();
}
var noop = function noop() {};
var tabControlTypes = [];
function isSectionField(field) {
  return field.section !== undefined;
}
function isSeparatorField(field) {
  return field.separator !== undefined;
}
function isControlField(field) {
  try {
    return !isTabbedField(field) && getControl(field.type) !== undefined;
  } catch (_e) {
    return false;
  }
}
function isTabbedField(field) {
  return isTabControl(field.type);
}
function isArrayOfTabs(v) {
  return Dialog.hasTabs(v);
}
function isTabControl(type) {
  return 'string' === typeof type && tabControlTypes.indexOf(type) !== -1;
}
var controls = new Map();
function getControl(type) {
  if (controls.has(type)) {
    return controls.get(type);
  } else {
    throw new Error("Control '".concat(type, "' is not registered as the Dialog control!"));
  }
}

// TODO: in future, different tab types may be supported, but for a single form only only type of tabs should be used
function checkTabSpecCorrectness(fields) {
  var topLevelTabs = fields.filter(isTabbedField);
  var preamble = "Invalid prop '".concat(fields, "' passed to '").concat(DFDialog.name || 'DFDialog', "'.");
  if (0 < topLevelTabs.length && topLevelTabs.length < fields.length) {
    return new Error("".concat(preamble, " No top level real fields are allowed if there are tab fields on top-level."));
  }
  var multipleTabCount = topLevelTabs.filter(function (_ref2) {
    var multiple = _ref2.multiple;
    return multiple;
  }).length;
  if (multipleTabCount > 1) {
    return new Error("".concat(preamble, " It is forbidden to have more than one multiple tab fields."));
  }
  return undefined;
}
function checkFieldsSpec(props) {
  var error = checkTabSpecCorrectness(props.fields);
  if (error) {
    console.error('Tab specification is wrong, please recheck provided `field` property', error);
  }
}
var Dialog = /*#__PURE__*/function (_Component) {
  _inherits(Dialog, _Component);
  var _super = _createSuper(Dialog);
  function Dialog(props) {
    var _this;
    _classCallCheck(this, Dialog);
    _this = _super.call(this, props);
    _this.state = {
      tabsCount: 1,
      activeTabId: undefined,
      verticalTabs: undefined
    };
    _this.form = void 0;
    _this.onClose = function (_event, reason) {
      if (reason === 'outsideClick') {
        return;
      }
      _this.setState({
        activeTabId: undefined
      });
      var _this$props = _this.props,
        _this$props$onClose = _this$props.onClose,
        onClose = _this$props$onClose === void 0 ? Dialog.defaultProps.onClose : _this$props$onClose,
        disableFormReset = _this$props.disableFormReset;
      onClose(_this.form);
      if (!disableFormReset) {
        window.setTimeout(function () {
          _this.form.reset();
        }, 0);
      }
    };
    _this.onApply = function () {
      var onAdd = _this.props.onAdd;
      return onAdd(_this.form).then(function (response) {
        var _ref3 = response || {},
          validationErrors = _ref3.validationErrors;
        if (validationErrors) {
          return validationErrors;
        }
        _this.onClose();
        return null;
      })["catch"](function (error) {
        return console.error(error);
      });
    };
    _this.isActiveTab = function (tabValues) {
      return getTabId(tabValues) === _this.state.activeTabId;
    };
    _this.onActiveTabChange = function (tab) {
      var onActiveTabChange = _this.props.onActiveTabChange;
      _this.setState({
        activeTabId: tab
      });
      if (typeof onActiveTabChange === 'function') {
        onActiveTabChange(tab);
      }
    };
    _this.renderTabs = function (_ref4) {
      var _tabbedFields$;
      var form = _ref4.form,
        formState = _objectWithoutProperties(_ref4, _excluded);
      var _this$state = _this.state,
        activeTabId = _this$state.activeTabId,
        verticalTabs = _this$state.verticalTabs;
      var _this$props2 = _this.props,
        size = _this$props2.size,
        fields = _this$props2.fields;
      var _this$prepareTabs = _this.prepareTabs(form, formState),
        tabItems = _this$prepareTabs.tabItems,
        removeTab = _this$prepareTabs.removeTab,
        createTab = _this$prepareTabs.createTab;
      var tabbedFields = fields;
      var TabComponent = getControl((_tabbedFields$ = tabbedFields[0]) === null || _tabbedFields$ === void 0 ? void 0 : _tabbedFields$.type);
      return /*#__PURE__*/_jsx(TabComponent, {
        className: bDialog('tabs-control', {
          vertical: Boolean(verticalTabs)
        }),
        size: size,
        tabItems: tabItems,
        activeTab: activeTabId,
        setActiveTab: _this.onActiveTabChange,
        onRemoveTab: removeTab,
        onCreateTab: createTab
      });
    };
    _this.renderDialogContent = function (handleSubmit, form) {
      _this.form = form;
      var _this$props3 = _this.props,
        waitingMessage = _this$props3.waitingMessage,
        headerProps = _this$props3.headerProps,
        footerProps = _this$props3.footerProps,
        fields = _this$props3.fields,
        formId = _this$props3.formId,
        size = _this$props3.size;
      var verticalTabs = _this.state.verticalTabs;
      var _ref5 = headerProps || {},
        title = _ref5.title,
        insertAfter = _ref5.insertAfter,
        insertBefore = _ref5.insertBefore;
      var _Dialog$defaultProps$ = Dialog.defaultProps.footerProps,
        textApplyDefault = _Dialog$defaultProps$.textApply,
        textCancelDefault = _Dialog$defaultProps$.textCancel;
      var _ref6 = footerProps || {},
        _ref6$textApply = _ref6.textApply,
        textApply = _ref6$textApply === void 0 ? textApplyDefault : _ref6$textApply,
        _ref6$textCancel = _ref6.textCancel,
        textCancel = _ref6$textCancel === void 0 ? textCancelDefault : _ref6$textCancel,
        content = _ref6.content,
        _ref6$propsButtonCanc = _ref6.propsButtonCancel,
        propsButtonCancel = _ref6$propsButtonCanc === void 0 ? {} : _ref6$propsButtonCanc,
        _ref6$propsButtonAppl = _ref6.propsButtonApply,
        propsButtonApply = _ref6$propsButtonAppl === void 0 ? {} : _ref6$propsButtonAppl,
        hidden = _ref6.hidden;
      var formState = form.getState();
      var disabled = _this.isApplyDisabled(formState);
      return /*#__PURE__*/_jsxs(Fragment, {
        children: [title && /*#__PURE__*/_jsx(CommonDialog.Header, {
          caption: /*#__PURE__*/_jsx("span", {
            className: bDialog('caption'),
            children: title
          }),
          insertAfter: insertAfter,
          insertBefore: insertBefore
        }), /*#__PURE__*/_jsxs("form", {
          id: formId,
          onSubmit: handleSubmit,
          children: [/*#__PURE__*/_jsx(CommonDialog.Body, {
            className: bDialog('body', {
              with_vertical_tabs: Boolean(verticalTabs)
            }),
            children: /*#__PURE__*/_jsx("div", {
              className: bDialog({
                size: size,
                with_vertical_tabs: Boolean(verticalTabs)
              }),
              children: isArrayOfTabs(fields) ? _this.renderTabbedFields() : /*#__PURE__*/_jsxs(React.Fragment, {
                children: [_this.renderFields(fields), _this.renderFormError()]
              })
            })
          }), !hidden && /*#__PURE__*/_jsx(CommonDialog.Footer, {
            preset: "default",
            loading: formState.submitting,
            textButtonCancel: textCancel,
            textButtonApply: textApply,
            propsButtonApply: _objectSpread({
              disabled: disabled,
              type: 'submit'
            }, propsButtonApply),
            propsButtonCancel: propsButtonCancel,
            onClickButtonCancel: _this.onClose,
            onClickButtonApply: noop,
            children: /*#__PURE__*/_jsx("div", {
              className: bDialog('footer-controls'),
              children: content
            })
          })]
        }), waitingMessage && /*#__PURE__*/_jsx("div", {
          className: bDialog('waiting-message'),
          children: /*#__PURE__*/_jsxs("div", {
            className: bDialog('waiting-message-inner'),
            children: [/*#__PURE__*/_jsx(InfoIcon, {}), waitingMessage]
          })
        })]
      });
    };
    _this.renderPageDialog = function (_ref7) {
      var handleSubmit = _ref7.handleSubmit,
        form = _ref7.form;
      var _this$props4 = _this.props,
        className = _this$props4.className,
        modal = _this$props4.modal,
        size = _this$props4.size;
      return /*#__PURE__*/_jsx("div", {
        className: bPage('wrapper', {
          modal: modal
        }, className),
        children: /*#__PURE__*/_jsx("div", {
          className: bPage({
            size: size
          }),
          children: _this.renderDialogContent(handleSubmit, form)
        })
      });
    };
    _this.renderModalDialog = function (_ref8) {
      var handleSubmit = _ref8.handleSubmit,
        form = _ref8.form;
      var _this$props5 = _this.props,
        visible = _this$props5.visible,
        className = _this$props5.className,
        size = _this$props5.size;
      return visible && /*#__PURE__*/_jsx(CommonDialog, {
        className: className,
        open: true,
        hasCloseButton: true,
        size: size,
        onClose: _this.onClose,
        children: _this.renderDialogContent(handleSubmit, form)
      });
    };
    _this.getFieldsCalculator = function () {
      var calculator = Dialog.collectSubscribers('', _this.props.fields).map(function (_ref9) {
        var name = _ref9.name,
          subscribers = _ref9.subscribers;
        return {
          field: name,
          updates: subscribers
        };
      });
      if (!calculator.length) {
        return undefined;
      }
      return createDecorator.apply(void 0, _toConsumableArray(calculator));
    };
    checkFieldsSpec(props);
    return _this;
  }
  _createClass(Dialog, [{
    key: "componentDidUpdate",
    value: function componentDidUpdate(_prevProps, prevState) {
      if (prevState.tabsCount !== this.state.tabsCount) {
        setTimeout(function () {
          var _document$querySelect;
          return (_document$querySelect = document.querySelector('.yc-dialog__modal')) === null || _document$querySelect === void 0 ? void 0 : _document$querySelect.scrollTo(0, 0);
        }, 0); // setTimeout for safari
      }
    }
  }, {
    key: "_isApplyDisabled",
    value: function _isApplyDisabled(formState) {
      var pristineSubmittable = this.props.pristineSubmittable;
      var pristine = formState.pristine,
        invalid = formState.invalid,
        hasSubmitErrors = formState.hasSubmitErrors,
        dirtySinceLastSubmit = formState.dirtySinceLastSubmit,
        hasValidationErrors = formState.hasValidationErrors;
      if (invalid) {
        return hasValidationErrors || hasSubmitErrors && !dirtySinceLastSubmit;
      }
      return pristineSubmittable ? false : pristine;
    }
  }, {
    key: "isApplyDisabled",
    value: function isApplyDisabled(formState) {
      var isApplyDisabled = this.props.isApplyDisabled;
      if (typeof isApplyDisabled === 'function') {
        return isApplyDisabled(formState);
      } else {
        return this._isApplyDisabled(formState);
      }
    }
  }, {
    key: "renderFieldGroup",
    value: function renderFieldGroup(field) {
      var _this2 = this;
      var fieldPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var name = field.name,
        type = field.type,
        className = field.className,
        warning = field.warning,
        fullWidth = field.fullWidth;
      var fieldName = fieldPrefix ? "".concat(fieldPrefix, ".").concat(name) : name;
      var required = field.required,
        validator = field.validator,
        _field$validateFields = field.validateFields,
        validateFields = _field$validateFields === void 0 ? [] : _field$validateFields,
        initialValue = field.initialValue,
        onChange = field.onChange;
      var validators = [];
      if (required) {
        validators.push(Dialog.makeRequiredValidator(field));
      }
      if (typeof validator === 'function') {
        validators.push(validator);
      }
      var _getControl = getControl(field.type),
        format = _getControl.format,
        customValidate = _getControl.validate,
        isEqual = _getControl.isEqual;
      if (customValidate) {
        validators.push(customValidate);
      }
      var validate = composeValidators.apply(void 0, validators);
      return /*#__PURE__*/_jsx(Field, {
        name: fieldName,
        initialValue: initialValue,
        validate: validate,
        validateFields: validateFields,
        isEqual: isEqual,
        format: format,
        formatOnBlur: true,
        children: function children(_ref10) {
          var input = _ref10.input,
            meta = _ref10.meta;
          return /*#__PURE__*/_jsxs("div", {
            className: bDialog('field-group', {
              type: type,
              'full-width': fullWidth,
              dirty: meta.dirty
            }, className),
            children: [Dialog.renderLabel(field), /*#__PURE__*/_jsxs("div", {
              className: bDialog('field-wrapper', {
                type: type,
                empty: Dialog.isEmptyField(field, input.value),
                'full-width': fullWidth
              }),
              children: [Dialog.renderField(field, input, meta), warning && /*#__PURE__*/_jsxs("div", {
                className: bDialog('field-warning'),
                children: [/*#__PURE__*/_jsx(WarningIcon, {}), /*#__PURE__*/_jsx("span", {
                  className: bDialog('field-warning-content'),
                  children: warning
                })]
              })]
            }), typeof onChange === 'function' && /*#__PURE__*/_jsx(OnChange, {
              name: fieldName,
              children: function children(value, oldValue) {
                if (value !== oldValue) {
                  onChange(value, oldValue, field, fieldName, _this2.form, _this2.form.getState());
                }
              }
            })]
          });
        }
      }, fieldName);
    }
  }, {
    key: "renderSection",
    value: function renderSection(field, index, fieldPrefix) {
      var _field$wrapTo = field.wrapTo,
        wrapTo = _field$wrapTo === void 0 ? function (sectionNode) {
          return sectionNode;
        } : _field$wrapTo,
        section = field.section,
        collapsible = field.collapsible,
        initialCollapsed = field.initialCollapsed;
      var children;
      if (!Array.isArray(field.fields) || field.fields.length === 0) {
        children = [];
      } else {
        children = this.renderFields(field.fields, fieldPrefix);
      }
      var key = "section-field-".concat(index);
      return /*#__PURE__*/_jsx(React.Fragment, {
        children: wrapTo( /*#__PURE__*/_jsx(CollapsibleSection, {
          className: bDialog('section'),
          caption: section,
          captionClassName: bDialog('section-title'),
          initialCollapsed: initialCollapsed,
          alwaysExpanded: !collapsible,
          children: children
        }))
      }, key);
    }
  }, {
    key: "renderFields",
    value: function renderFields(fields, fieldPrefix) {
      var _this3 = this;
      return fields.map(function (field, index) {
        if (isSectionField(field)) {
          return _this3.renderSection(field, index, fieldPrefix);
        } else if (isSeparatorField(field)) {
          return Dialog.renderSeparator(field, index);
        } else {
          var visibilityCondition = field.visibilityCondition;
          if (visibilityCondition) {
            return /*#__PURE__*/_jsx(Condition, _objectSpread(_objectSpread({}, visibilityCondition), {}, {
              children: _this3.renderFieldGroup(field, fieldPrefix)
            }), "cond-".concat(index));
          }
          return _this3.renderFieldGroup(field, fieldPrefix);
        }
      });
    }
  }, {
    key: "renderFormError",
    value: function renderFormError() {
      var _ref11 = this.form.getState().submitErrors || {},
        formError = _ref11[FORM_ERROR];
      if (!formError) {
        return null;
      }
      return /*#__PURE__*/_jsx("div", {
        className: bDialog('error-message'),
        children: formError
      });
    }
  }, {
    key: "renderMultiTabContent",
    value: function renderMultiTabContent(tab) {
      var _this4 = this;
      var virtualized = this.props.virtualized;
      var name = tab.name,
        fields = tab.fields,
        renderControls = tab.renderControls;
      return /*#__PURE__*/_jsx(FieldArray, {
        name: name,
        children: function children(_ref12) {
          var tabs = _ref12.fields;
          return tabs.map(function (fieldPrefix, index) {
            var tabValues = tabs.value[index];
            var isActive = _this4.isActiveTab(tabValues);
            var onCreate = function onCreate(active) {
              var formState = _this4.form.getState();
              var _this4$prepareTabs = _this4.prepareTabs(_this4.form, formState),
                createTab = _this4$prepareTabs.createTab;
              createTab(name, active);
            };
            var onRemove = function onRemove() {
              var formState = _this4.form.getState();
              var _this4$prepareTabs2 = _this4.prepareTabs(_this4.form, formState),
                removeTab = _this4$prepareTabs2.removeTab;
              removeTab(_objectSpread(_objectSpread({}, tabValues), {}, {
                name: name
              }));
            };
            return (isActive || !virtualized) && /*#__PURE__*/_jsxs("div", {
              className: bDialog('tabs-content-item', {
                hidden: !isActive,
                name: name
              }),
              children: [_this4.renderFields(fields, fieldPrefix), _this4.renderFormError(), renderControls && /*#__PURE__*/_jsx("div", {
                className: bDialog('extra-controls'),
                children: renderControls(tabValues, onCreate, index !== 0 ? onRemove : undefined)
              })]
            }, index);
          });
        }
      }, name);
    }
  }, {
    key: "renderSingleTabContent",
    value: function renderSingleTabContent(tab) {
      var virtualized = this.props.virtualized;
      var activeTabId = this.state.activeTabId;
      var name = tab.name,
        fields = tab.fields;
      var isActive = name === activeTabId;
      return (isActive || !virtualized) && /*#__PURE__*/_jsxs("div", {
        className: bDialog('tabs-content-item', {
          hidden: !isActive,
          name: name
        }),
        children: [this.renderFields(fields, name), this.renderFormError()]
      }, "single-tab-".concat(name));
    }
  }, {
    key: "isVisibleTab",
    value: function isVisibleTab(tabAttributes, values) {
      var visibilityCondition = tabAttributes.visibilityCondition;
      if (!visibilityCondition) {
        return true;
      }
      var value = _get(values, visibilityCondition.when);
      return visibilityCondition.isActive(value);
    }
  }, {
    key: "prepareTabs",
    value: function prepareTabs(form, formState) {
      var _this5 = this;
      var _form$mutators = form.mutators,
        remove = _form$mutators.remove,
        push = _form$mutators.push;
      var activeTabId = this.state.activeTabId;
      var propFields = this.props.fields;
      var stateValues = formState.values,
        errors = formState.errors;
      var tabItems = [];
      var firstMultiTabIndex = 0;
      var values = stateValues;
      var fields = propFields;
      var createTab = function createTab(fieldName, active) {
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var tabSpec = _find(fields, function (fieldSpec) {
          return fieldSpec.name === fieldName;
        });
        var _ref13 = tabSpec || {},
          onCreateTab = _ref13.onCreateTab;
        if (!(tabSpec !== null && tabSpec !== void 0 && tabSpec.multiple)) {
          return;
        }
        var userOptions = options.userOptions;
        var index = _findIndex(values[fieldName], _this5.isActiveTab);
        var _values$fieldName = values[fieldName][index === -1 ? 0 : index],
          _id = _values$fieldName.id,
          srcTabData = _objectWithoutProperties(_values$fieldName, _excluded2);
        var newTabData = onCreateTab ? onCreateTab(srcTabData, {
          userOptions: userOptions
        }) : srcTabData;
        if (newTabData.id === undefined) {
          var ids = new Set(_map(values[fieldName], function (_ref14) {
            var id = _ref14.id;
            return String(id);
          }));
          for (var i = 0; i <= ids.size; ++i) {
            var id = String(i + 1);
            if (!ids.has(id)) {
              newTabData.id = id;
              break;
            }
          }
        } else {
          checkTabId(newTabData, values[fieldName]);
        }
        push(fieldName, newTabData);
        _this5.setState({
          tabsCount: tabItems.length + 1
        });
        if (active) {
          _this5.onActiveTabChange(getTabId(newTabData));
        }
      };
      var removeTab = function removeTab(tabItemToDelete) {
        var tabIndexToDelete = _findIndex(tabItems, function (item) {
          return item.id === tabItemToDelete.id;
        });
        if (tabIndexToDelete < 0) {
          return;
        }
        if (tabItemToDelete.id === activeTabId) {
          var tabIdToSwitch = tabItems[tabIndexToDelete - 1].id;
          _this5.setState({
            activeTabId: tabIdToSwitch
          });
        }
        remove(tabItemToDelete.name, tabIndexToDelete - firstMultiTabIndex);
        _this5.setState({
          tabsCount: tabItems.length - 1
        });
      };
      var seenMultiTab = false;
      var updateActiveTab = false;
      Dialog.operateOnTabs({
        fields: fields,
        values: values,
        errors: errors
      }, function (tabAttributes) {
        var id = tabAttributes.id,
          error = tabAttributes.error,
          name = tabAttributes.name,
          multiple = tabAttributes.multiple,
          index = tabAttributes.index,
          renderControls = tabAttributes.renderControls,
          title = tabAttributes.title,
          isRemovable = tabAttributes.isRemovable;
        if (!_this5.isVisibleTab(tabAttributes, stateValues)) {
          updateActiveTab = updateActiveTab || activeTabId === id;
          return;
        }
        if (multiple && !seenMultiTab) {
          seenMultiTab = true;
          firstMultiTabIndex = index;
        }
        tabItems.push(_objectSpread(_objectSpread({
          id: id,
          error: error,
          name: name,
          multiple: multiple
        }, {
          renderControls: renderControls
        }), {}, {
          title: title,
          removable: isRemovable(index - firstMultiTabIndex)
        }));
      });
      if (updateActiveTab) {
        var _tabItems$;
        this.setState({
          activeTabId: (_tabItems$ = tabItems[0]) === null || _tabItems$ === void 0 ? void 0 : _tabItems$.id
        });
      }
      return {
        tabItems: tabItems,
        createTab: createTab,
        removeTab: removeTab
      };
    }
  }, {
    key: "renderTabbedFields",
    value: function renderTabbedFields() {
      var _this6 = this;
      var fields = this.props.fields;
      var verticalTabs = this.state.verticalTabs;
      var tabbedFields = fields;
      return /*#__PURE__*/_jsxs(Fragment, {
        children: [/*#__PURE__*/_jsx(FormSpy, {
          render: this.renderTabs,
          subscription: {
            values: true,
            errors: true
          }
        }), /*#__PURE__*/_jsx("div", {
          className: bDialog('tabs-content', {
            with_vertical_tabs: verticalTabs
          }),
          children: tabbedFields.map(function (tab) {
            return tab.multiple ? _this6.renderMultiTabContent(tab) : _this6.renderSingleTabContent(tab);
          })
        })]
      });
    }
  }, {
    key: "hasTabs",
    value: function hasTabs() {
      return Dialog.hasTabs(this.props.fields);
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props6 = this.props,
        initialValues = _this$props6.initialValues,
        modal = _this$props6.modal,
        formExtras = _this$props6.formExtras,
        validate = _this$props6.validate;
      var defaultValues = this.state.defaultValues;
      var calculator = this.getFieldsCalculator();
      var decorators = calculator ? [calculator] : undefined;
      var values;
      if (initialValues) {
        // simple form initialized with values
        values = Object.assign({}, defaultValues, initialValues);
      } else {
        // simple form with default field values
        values = defaultValues;
      }
      var renderDialog = modal ? this.renderModalDialog : this.renderPageDialog;
      var _ref15 = formExtras !== null && formExtras !== void 0 ? formExtras : {},
        mutators = _ref15.mutators;
      return /*#__PURE__*/_jsx(Form, _objectSpread(_objectSpread({
        initialValuesEqual: _isEqual,
        keepDirtyOnReinitialize: true
      }, formExtras), {}, {
        onSubmit: this.onApply,
        validate: validate,
        subscription: Dialog.FINAL_FORM_SUBSCRIPTIONS,
        mutators: this.hasTabs() ? _objectSpread(_objectSpread({}, mutators), arrayMutators) : mutators,
        initialValues: values,
        decorators: decorators,
        render: renderDialog
      }));
    }
  }], [{
    key: "registerControl",
    value:
    /**
     * @param {string} controlId
     * @param {function} controlComponent
     * @param {{
     *     suppressDuplicateWarning: boolean | undefined,
     * } | undefined} opts
     */
    function registerControl(controlId, controlComponent) {
      var _ref16 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        suppressDuplicateWarning = _ref16.suppressDuplicateWarning;
      if (controls.has(controlId) && !suppressDuplicateWarning) {
        console.warn("Control '".concat(controlId, "' is already registered as the Dialog control!"));
      }
      if (controlComponent.isTabControl) {
        tabControlTypes.push(controlId);
      }
      controls.set(controlId, controlComponent);
    }
  }, {
    key: "registerTabControl",
    value: function registerTabControl(controlId, tabComponent, opts) {
      Dialog.registerControl(controlId, tabComponent, opts);
    }
  }, {
    key: "isEmptyField",
    value: function isEmptyField(field, value) {
      var control = getControl(field.type);
      var isEmpty = control && control.isEmpty ? control.isEmpty : function () {
        return false;
      };
      return isEmpty(value);
    }
  }, {
    key: "getDefaultValues",
    value: function getDefaultValues() {
      var fields = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      return _reduce(fields, function (acc, item) {
        var field = item;
        if (isSectionField(field)) {
          if (Array.isArray(field.fields)) {
            return _objectSpread(_objectSpread({}, acc), Dialog.getDefaultValues(field.fields));
          }
        } else if (isTabbedField(field)) {
          var tabData = Dialog.getDefaultValues(field.fields);
          acc[field.name] = field.multiple ? [_objectSpread(_objectSpread({}, tabData), {}, {
            id: 1
          })] : tabData;
        } else if (isControlField(field)) {
          acc[field.name] = getControl(field.type).getDefaultValue();
        }
        return acc;
      }, {});
    }
  }, {
    key: "makeRequiredValidator",
    value: function makeRequiredValidator(field) {
      return function (value) {
        return Dialog.isEmptyField(field, value) ? i18n('validator:value-required') : undefined;
      };
    }
  }, {
    key: "getDerivedStateFromProps",
    value: function getDerivedStateFromProps(props, prevState) {
      var res = {};
      var fields = props.fields;
      if (!prevState.defaultValues) {
        res.defaultValues = Dialog.getDefaultValues(fields);
      }
      if (prevState.verticalTabs === undefined) {
        res.verticalTabs = Dialog.getTabsOrientation(fields);
      }
      if (Dialog.hasTabs(fields)) {
        var initialValues = props.initialValues;
        if (prevState.activeTabId === undefined) {
          res.activeTabId = Dialog.getFirstTabId(fields, initialValues);
        }
      }
      return _isEmpty(res) ? null : res;
    }
  }, {
    key: "hasTabs",
    value: function hasTabs(fields) {
      return fields.some(isTabbedField);
    }
  }, {
    key: "getFirstTabId",
    value: function getFirstTabId(fields, values) {
      var _fields$ = fields[0],
        multiple = _fields$.multiple,
        name = _fields$.name;
      return multiple ? getTabId(values[name][0]) : name;
    }
  }, {
    key: "getTabsOrientation",
    value: function getTabsOrientation(fields) {
      var _fields = _slicedToArray(fields, 1),
        field = _fields[0];
      if (!field || !isTabbedField(field)) {
        return null;
      }
      var control = getControl(field.type);
      var isTabControl = control.isTabControl,
        isTabControlVertical = control.isTabControlVertical;
      if (isTabControl === undefined) {
        return null;
      }
      return isTabControlVertical;
    }
  }, {
    key: "renderField",
    value: function renderField(field, input, meta) {
      var extras = field.extras;
      if (typeof extras === 'function') {
        return /*#__PURE__*/_jsx(FormSpy, {
          subscription: {
            values: true
          },
          children: function children(_ref17) {
            var values = _ref17.values,
              form = _ref17.form;
            var extrasValues = extras(values, {
              input: input,
              form: form,
              field: field
            });
            return /*#__PURE__*/_jsx(FieldWithExtras, {
              field: field,
              input: input,
              meta: meta,
              extras: extrasValues
            });
          }
        });
      }
      return /*#__PURE__*/_jsx(FieldWithExtras, {
        field: field,
        input: input,
        meta: meta,
        extras: extras
      });
    }
  }, {
    key: "renderSeparator",
    value: function renderSeparator(_field, index) {
      return /*#__PURE__*/_jsx("div", {
        className: bDialog('separator')
      }, "separator-".concat(index));
    }
  }, {
    key: "renderLabel",
    value: function renderLabel(field) {
      var name = field.name,
        type = field.type,
        required = field.required,
        caption = field.caption,
        tooltip = field.tooltip;
      var renderLabel = controls.get(type) && controls.get(type).renderLabel;
      if (typeof renderLabel === 'function') {
        return renderLabel(field);
      }
      return /*#__PURE__*/_jsxs("label", {
        className: bDialog('label'),
        htmlFor: name,
        children: [/*#__PURE__*/_jsx("div", {
          className: bDialog('label-text', {
            required: required
          }),
          children: caption
        }), Boolean(tooltip) && /*#__PURE__*/_jsx("div", {
          children: /*#__PURE__*/_jsx(Popover, {
            content: tooltip,
            delayOpening: 150,
            delayClosing: 150,
            placement: ['bottom', 'top', 'right'],
            children: /*#__PURE__*/_jsx(TooltipIcon, {
              className: bDialog('q-icon')
            })
          })
        })]
      });
    }
  }, {
    key: "operateOnTabs",
    value: function operateOnTabs(_ref18, callback) {
      var fields = _ref18.fields,
        values = _ref18.values,
        errors = _ref18.errors;
      var tabIndex = 0;
      fields.forEach(function (field) {
        var name = field.name;
        var type = field.type,
          multiple = field.multiple,
          renderControls = field.renderControls,
          _field$title = field.title,
          title = _field$title === void 0 ? name : _field$title,
          getTitle = field.getTitle,
          isRemovable = field.isRemovable,
          visibilityCondition = field.visibilityCondition;
        if (!isTabControl(type)) {
          throw new Error("Expected field of type 'tab', got '".concat(type, "'"));
        }
        var fieldValues = values[name];
        var fieldErrors = errors && errors[name];
        var args = {
          multiple: multiple,
          name: name,
          renderControls: renderControls,
          visibilityCondition: visibilityCondition
        };
        if (multiple) {
          fieldValues.forEach(function (tabValues, index) {
            var id = getTabId(tabValues);
            callback(_objectSpread({
              id: id,
              error: fieldErrors && fieldErrors[index],
              index: tabIndex,
              title: typeof getTitle === 'function' ? getTitle(tabValues) : "".concat(title, " ").concat(id),
              isRemovable: typeof isRemovable === 'function' ? isRemovable : function (index) {
                return index !== 0;
              }
            }, args));
            tabIndex++;
          });
        } else {
          callback(_objectSpread({
            id: name,
            error: fieldErrors,
            index: tabIndex,
            title: typeof getTitle === 'function' ? getTitle(fieldValues) : title,
            isRemovable: function isRemovable() {
              return false;
            }
          }, args));
          tabIndex++;
        }
      });
    }
  }, {
    key: "collectSubscribers",
    value: function collectSubscribers() {
      var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
      var fields = arguments.length > 1 ? arguments[1] : undefined;
      var dst = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
      return _reduce(fields, function (acc, field) {
        var item = field;
        if (isSectionField(item)) {
          if (Array.isArray(item.fields)) {
            Dialog.collectSubscribers(prefix, item.fields, acc);
          }
        } else {
          var name = item.name,
            subscribers = item.subscribers;
          var _fullName = "".concat(prefix).concat(name);
          if (subscribers) {
            acc.push({
              name: _fullName,
              subscribers: subscribers
            });
          }
          if (isTabbedField(item)) {
            Dialog.collectSubscribers("".concat(_fullName, "."), item.fields, acc);
          }
        }
        return acc;
      }, dst);
    }
  }]);
  return Dialog;
}(Component);
Dialog.defaultProps = {
  pristineSubmittable: false,
  modal: true,
  size: 'm',
  headerProps: {},
  footerProps: {
    textApply: 'Confirm',
    textCancel: 'Cancel'
  },
  onClose: function onClose() {}
};
Dialog.FINAL_FORM_SUBSCRIPTIONS = {
  submitting: true,
  pristine: true,
  invalid: true,
  dirtySinceLastSubmit: true,
  hasValidationErrors: true,
  hasSubmitErrors: true
};
Dialog.registerTabControl('tab', TabField);
Dialog.registerTabControl('tab-vertical', TabFieldVertical);
Dialog.registerControl('text', TextControl);
Dialog.registerControl('textarea', TextAreaControl);
Dialog.registerControl('select', SelectControl);
Dialog.registerControl('tumbler', TumblerControl);
Dialog.registerControl('block', CustomBlock);
Dialog.registerControl('plain', PlainText);
Dialog.registerControl('checkbox', CheckBoxControl);
Dialog.registerControl('radio', RadioButtonControl);
Dialog.registerControl('editable-list', EditableList);
Dialog.registerControl('multi-editable-lists', EditableManyLists);
Dialog.registerControl('multi-text', MultiTextControl);
export var DeprecatedDialog = Dialog;
export function DFDialog(props) {
  return /*#__PURE__*/_jsx(Dialog, _objectSpread({}, props));
}
export var registerDialogControl = Dialog.registerControl;
export var registerDialogTabControl = Dialog.registerTabControl;
function FieldWithExtras(_ref19) {
  var field = _ref19.field,
    _ref19$extras = _ref19.extras,
    extras = _ref19$extras === void 0 ? {} : _ref19$extras,
    meta = _ref19.meta,
    input = _ref19.input;
  var type = field.type,
    name = field.name,
    touched = field.touched;
  var error;
  if (meta.error && (touched || meta.touched || meta.dirty)) {
    error = meta.error;
  } else if (meta.submitError && !meta.dirtySinceLastSubmit && !meta.submitting) {
    error = meta.submitError;
  }
  var onFocusExternal = extras.onFocus,
    onBlurExternal = extras.onBlur;
  var onFocusField = input.onFocus,
    onBlurField = input.onBlur;
  var onFocus = useStableEventHandler(applyFunctions(onFocusExternal, onFocusField));
  var onBlur = useStableEventHandler(applyFunctions(onBlurExternal, onBlurField));
  var controlProps = _objectSpread(_objectSpread(_objectSpread({
    id: name
  }, extras), input), {}, {
    onFocus: onFocus,
    onBlur: onBlur
  });
  var Control = getControl(type);
  if (Control.hasErrorRenderer) {
    controlProps.error = error;
    error = null;
  }
  return /*#__PURE__*/_jsxs(React.Fragment, {
    children: [/*#__PURE__*/React.createElement(Control, _objectSpread({}, controlProps)), error && /*#__PURE__*/_jsx("div", {
      className: bDialog('field-error'),
      children: error
    })]
  });
}
// #sourceMappingURL=Dialog.js.map